// Copyright 2022 ESRI
//
// All rights reserved under the copyright laws of the United States
// and applicable international laws, treaties, and conventions.
//
// You may freely redistribute and use this sample code, with or
// without modification, provided you include the original copyright
// notice and use restrictions.
//
// See the Sample code usage restrictions document for further information.
//
#ifndef DISPLAY_A_MAP_H
#define DISPLAY_A_MAP_H

namespace Esri::ArcGISRuntime {

class Map;
class MapQuickView;

class Graphic;
class GraphicsOverlay;
class PictureMarkerSymbol;
class RouteTask;

}

enum RouteBuilderStatus
{
  NotStarted,
  SelectedStart,
  SelectedStartAndEnd,
};

#include <QObject>

#include "RouteParameters.h"
class QAbstractListModel;
Q_MOC_INCLUDE("QAbstractListModel")
Q_MOC_INCLUDE("MapQuickView.h")

class Display_a_map : public QObject
{
    Q_OBJECT
    Q_PROPERTY(Esri::ArcGISRuntime::MapQuickView* mapView READ mapView WRITE setMapView NOTIFY mapViewChanged)

    Q_PROPERTY(QAbstractListModel* directions MEMBER m_directions NOTIFY directionsChanged)

public:
    explicit Display_a_map(QObject* parent = nullptr);
    ~Display_a_map() override;

signals:
    void mapViewChanged();

     void directionsChanged();

private:
    Esri::ArcGISRuntime::MapQuickView* mapView() const;
    void setMapView(Esri::ArcGISRuntime::MapQuickView* mapView);
    void setupViewpoint();

    void setupRouteTask();
    void findRoute();
    void resetState();

    Esri::ArcGISRuntime::Map* m_map = nullptr;
    Esri::ArcGISRuntime::MapQuickView* m_mapView = nullptr;

    Esri::ArcGISRuntime::GraphicsOverlay* m_graphicsOverlay = nullptr;
    Esri::ArcGISRuntime::RouteTask* m_routeTask = nullptr;
    Esri::ArcGISRuntime::Graphic* m_startGraphic = nullptr;
    Esri::ArcGISRuntime::Graphic* m_endGraphic = nullptr;
    Esri::ArcGISRuntime::Graphic* m_lineGraphic = nullptr;
    QAbstractListModel* m_directions = nullptr;
    Esri::ArcGISRuntime::RouteParameters m_routeParameters;
    RouteBuilderStatus m_currentState;

};

#endif // DISPLAY_A_MAP_H

